package solutions.bitbadger.documents.java.java.integration.common;

import solutions.bitbadger.documents.Field;
import solutions.bitbadger.documents.java.jvm.Count;
import solutions.bitbadger.documents.java.integration.ThrowawayDatabase;
import solutions.bitbadger.documents.java.java.testDocs.JsonDocument;

import java.util.List;
import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static solutions.bitbadger.documents.java.integration.TypesKt.TEST_TABLE;

/**
 * Integration tests for the `Count` object
 */
final public class CountFunctions {

    public static void all(ThrowawayDatabase db) {
        JsonDocument.load(db);
        assertEquals(5L, Count.all(TEST_TABLE, db.getConn()), "There should have been 5 documents in the table");
    }

    public static void byFieldsNumeric(ThrowawayDatabase db) {
        JsonDocument.load(db);
        assertEquals(3L, Count.byFields(TEST_TABLE, List.of(Field.between("numValue", 10, 20)), db.getConn()),
                "There should have been 3 matching documents");
    }

    public static void byFieldsAlpha(ThrowawayDatabase db) {
        JsonDocument.load(db);
        assertEquals(1L, Count.byFields(TEST_TABLE, List.of(Field.between("value", "aardvark", "apple")), db.getConn()),
                "There should have been 1 matching document");
    }

    public static void byContainsMatch(ThrowawayDatabase db) {
        JsonDocument.load(db);
        assertEquals(2L, Count.byContains(TEST_TABLE, Map.of("value", "purple"), db.getConn()),
                "There should have been 2 matching documents");
    }

    public static void byContainsNoMatch(ThrowawayDatabase db) {
        JsonDocument.load(db);
        assertEquals(0L, Count.byContains(TEST_TABLE, Map.of("value", "magenta"), db.getConn()),
                "There should have been no matching documents");
    }

    public static void byJsonPathMatch(ThrowawayDatabase db) {
        JsonDocument.load(db);
        assertEquals(2L, Count.byJsonPath(TEST_TABLE, "$.numValue ? (@ < 5)", db.getConn()),
                "There should have been 2 matching documents");
    }

    public static void byJsonPathNoMatch(ThrowawayDatabase db) {
        JsonDocument.load(db);
        assertEquals(0L, Count.byJsonPath(TEST_TABLE, "$.numValue ? (@ > 100)", db.getConn()),
                "There should have been no matching documents");
    }

    private CountFunctions() {
    }
}
